---
title: async-mysql documention

language_tabs:
    - javascript

includes:
    - errors

---
# Introduction

>以下案例默认引入

```javascript
  const database = require('async-mysql')
  const db = new database()
  let obj = {
    id:'1234',
    hasGirlFriend:'yes',
    name:'joker'
  }
  //所有db实例的方法均会返回一个promise实例
  db.insert('user_info',obj)
  .then(t=>console.log(t))
  .catch(e=>console.log(e))
```

使用`NodeJs8.0`的`util.promisify`和`async await`
解决`node-mysql`模块的进行数据库操作时陷入`callback hell`

<aside class="warning">
使用本模块进行各种数据库操作时，其语法必须同时满足<code>MySQL</code>语法和<code>node-mysql</code>语法（比如其中的各种占位符）
</aside>

# Async MySQL

## insert
>使用方法如下

```javascript
//插入数据
db.raw_insert([arr])
usage:
    db.raw_insert(['user_info',['id','name','hasGirlFriend'],['1234','joker','yes']])
    .then(t=>console.log(t))
    .catch(err=>console.log(err))
execude as:
    let sql =
    `insert into user_info (id,name,hasGirlFriend) values ('1234','joker','1234')`

    pool.query(sql,(e,r)=>console.log(r))

db.insert(tableName,opt)
usage:
    let opt = {
      id:'1234',
      name:'joker',
      hasGirlFriend:'yes'
    }
    db.insert('user_info',opt)
    .then(t=>console.log(t))
    .catch(e=>console.log(e))

```
####实例方法

  方法         |  参数   |  说明  |
--------------|--------|----------|
db.raw_insert | arr     |         |
db.insert | tableName,opt |       |

<aside success="note">
  两种方法的数据操作结果一致，只是语法不同而已。
</aside>


##select

```javascript
db.selectByParams(sql,[arr])

usage:
    let sql =
    `select ?? from ?? where name = ? and hasGirlFriend = ? `
    let opt = [['id','name','age','hasGirlFriend'],'user_info','joker','nope']
    db.selectByParams(sql,opt)
    .then((t)=>console.log(t))
    .catch(e=>console.log(e))


db.selectByPages(sql,[arr])
//此方法可与上述方法合并

usage:
    let sql = `select ?? from ?? where name = ? and hasGirlFriend = ? limit ?,?`
    //此类分页方法扫描数据表时过慢  TODO  
    let opt = [['id','name','age','hasGirlFriend'],'user_info','joker','nope',0,5]
    db.selectByPages(sql,opt)
    .then(t=>console.log(t))
    .catch(e=>console.log(e))

db.countTable(tableName)

usage:

    db.countTable('user_info')
    .then(t=>console.log(t))
    .catch(e=>console.log(e))

```
###实例方法

方法名             |  参数   |   说明   |
---------------------------|-------- |--------- |
db.selectByParams          | sql,opt |           |
<del>db.selectByPages</del> | sql,opt|与上述方法一样|
db.countTable              | tableName |         |




##update

```javascript
  db.update(arr)
  //arr = [tableName,obj,params]
  //适用一个过滤条件
  usage:
      let opt = {
        name:'joker',
        hasGirlFriend:'no'
      }
      let params = {
        id:'12345'
      }
      db.update('user_info',opt,params)
      .then(t=>console.log(t))
      .catch(e=>console.log(e))

  db.raw_update(sql,opt)
  //自定义更新语句 适用多个过滤条件的
  usage:
      let sql =
      `update ?? set ? where name = ? and hasGirlFriend = ?`
      let opt = {
        age:19,
        hasDream:'no'
      }
      let arr = ['user_info',opt,'joker','yes']
      db.update(sql,arr)
      .then(t=>console.log(t))
      .catch(e=>console.log(e))

```



##delete

```javascript

  db.delete(arr)
  usage:
      let arr = ["user_info",{name:'joker'}]
  execude as:
      sql = `delete from user_info where name = 'joker'`
      pool.query(sql,……)


  db.raw_delete(sql,arr)
  usage:
      let sql = `delete from ?? where name = ? and age = ? `
      let arr = ["user_info",'joker','22']
      db.delete(sql,arr)
      .then(t=>console.log(t))
      .catch(e=>console.log(e))
```
