const pool = require('./src/mysql.js').pool()
const util = require('util')

pool.on('release',(conn)=>{
	console.log('%d 线程已释放', conn.threadId);
})

//原始查询方法 
function query(sql,params,cb){
	pool.getConnection((err,conn)=>{
		if(err){
			cb(err,null)
		}else{
			//防止异步操作某一段发生err 从而可以rollback()
			//只能回滚一条数据 两段式数据如何回滚 TODO
			conn.beginTransaction((err)=>{
				if(err){
					cb(err)
				}else{
					conn.query(sql,params,(err,rows)=>{					
						if(err){
							cb(err,null)
							return conn.rollback(err=>{
								console.log('rollback err',err)
							})
						}else{
							conn.commit(err=>{
								
								if(err){
									cb(err,null)
									return conn.rollback(err=>{
										console.log('rollback err',err)
									})
									//console.log('commit err',err)
								}else{
									cb(null,rows)
									conn.release()//放回连接池
									console.log('mysql commit success')
								}
							})							
						}			
					})
				}
			})			
		}		
	})
}

//将query函数转为一个promise，配合async await服用更佳
const async_query = util.promisify(query)

//判断增删改操作结果
const checkAffected = (rows)=>{
		//console.log('check rows:',rows)
		if(!rows || !rows.affectedRows){
			return {msg:'did not worked',affected:false,num:0}
		}else if(rows.affectedRows==0){
			return {msg:'nothing changed',affected:false,num:rows.affectedRows}
			//TODO
		}else if(rows.affectedRows==1){
			return {msg:'yes,1 row changed',affected:true,num:rows.affectedRows}
		}else{
			return {msg:`yes,${rows.affectedRows} changed`,affected:true,num:rows.affectedRows}
		}	
}

// 判断查询结果
const checkSelected = (rows)=>{
	if(!rows||rows.length==0){
		return {msg:'查询结果不存在',success:false,result:[],count:0}
	}else{
		return {msg:'查询结果正常',success:true,result:rows}
	}
}


//创建 database 类 ，此类里封装基本mysql操作
class database{
	/*
		此类内置方法都是分支条件较少的语句
		node-mysql对于查询条件较多的语句拼接支持比较有限
		函数化太困难
	*/

	//创建表格
	async createTable(sql,cb){
		let back = await query(sql,[])
		return back 
	}

	//无条件count 仅提供tableName即可
	async countTable(tableName,cb){
		let _sql = 'select count(*) as count from ??'
		let back = await async_query(_sql,tableName)
		let end = checkSelected(back)
		return end
	}

	//通过自定义sql语句和参数，查询多参数数据（废弃）
	async selectByParams(sql,arr,cb){
		let _sql = sql
		let back = await async_query(_sql,arr)
		let end = checkSelected(back)
		return end
	}
	//废弃
	async selectByPages(sql,arr,cb){
		let _sql = sql 
		let back = await async_query(_sql,arr)
		let end = checkSelected(back)
		return end
	}

	//传入sql语句 和参数数组 
	async raw_select(sql,arr,cb){
		let _sql = sql
		let back = await async_query(_sql,arr)
		let end = checkSelected(back)
		return end
	}
	//插入数据 mysql异类 我也不知道是从哪里抄来的
	async insert(tableName,obj,cb){
		let _sql = "insert into ?? set ?"	
		let back = await async_query(_sql,[tableName,obj])
		let end = checkAffected(back)
		return end
	}

	//原始插入数据方法
	async raw_insert(arr,cb){
		let _sql = "insert into ?? (??) values (?)"
		let back = await async_query(_sql,arr)
		let end = checkAffected(back)
		return end
	}

	//更新只有一个where条件的数据 
	//arr = [string,obj,obj]
	async update(arr,cb){
		let _sql = 'update ?? set ? where ?'
		let back = await async_query(_sql,arr)
		let end = checkAffected(back)
		return end 
	}

	//多个过滤条件 传入原始myql 和 参数
	async raw_update(sql,arr,cb){
		let _sql = sql
		let back = await async_query(_sql,arr)
		let end = checkAffected(back)
		return end
	}
	//删除数据 只有一个过滤条件
	//arr = [tableName,obj]
	async delete(arr,cb){
		let _sql = 'delete from ?? where ?'
		let back = await async_query(_sql,arr)
		let end = checkAffected(back)
		return end
	}

	//删除数据 多个过滤条件的
	//arr=[tableName,params1,params2]
	async raw_delete(sql,arr,cb){
		let _sql = sql
		let back = await async_query(_sql,arr)
		let end = checkAffected(back)
		return end
	}
}

module.exports = database

